define(["require", "exports", "jquery", "fecha", "seecool/utilities"], function (require, exports, $, fecha, utilities) {
    "use strict";
    var ShipTrackLoader = (function () {
        function ShipTrackLoader(url, ships, start, end, fnLoad, fnError, context, token, skipFun) {
            var encodedShips = [];
            for (var i = 0; i < ships.length; i++) {
                encodedShips[i] = ships[i].replace(/\-/g, '-0');
            }
            this.url_ = url;
            this.ships_ = encodedShips.join("-1");
            this.start_ = start;
            this.end_ = end;
            this.current_ = start;
            this.points = {};
            this.undeterminedPoints_ = {};
            this.ajax_ = null;
            this.fnLoad_ = fnLoad || null;
            this.fnError_ = fnError || null;
            this.context_ = context || window;
            this.token_ = token;
            this.skipFun_ = skipFun;
        }
        ;
        ShipTrackLoader.prototype.load = function (start, end) {
            var _this = this;
            var queryEndTime, queryStartTime;
            if (this.ajax_)
                return;
            if (arguments.length > 0) {
                queryEndTime = start;
                queryStartTime = end;
            }
            else {
                queryEndTime = new Date(this.current_);
                queryEndTime.setMinutes(this.current_.getMinutes() + 60);
                queryStartTime = this.current_;
            }
            this.current_ = queryEndTime;
            if (queryEndTime > this.end_)
                queryEndTime = this.end_;
            //var url = "/api/ShipTrack/ShipTrackRawQuery/";
            //var url = "api/shiphistory/ShipTrack/ShipTrackRawQuery/"+this.ships_+"/"+utilities.format(queryStartTime, 'yyyy-MM-dd HH:mm:ss')+"/"+utilities.format(queryEndTime, 'yyyy-MM-dd HH:mm:ss');
            var url = this.url_
                + "/ShipTrack/ShipTrackUnionQuery/" + this.ships_
                + "/" + fecha.format(queryStartTime, 'YYYY-MM-DD HH:mm:ss')
                + "/" + fecha.format(queryEndTime, 'YYYY-MM-DD HH:mm:ss');
            this.ajax_ = $.ajax({
                url: url,
                //data: {
                //    ShipIds: this.ships_,
                //    StartTime: utilities.format(queryStartTime, 'yyyy-MM-dd HH:mm:ss'),
                //    EndTime: utilities.format(queryEndTime, 'yyyy-MM-dd HH:mm:ss')
                //},
                dataType: 'json',
                type: 'get',
                context: this
            });
            this.ajax_.done(function (result) {
                _this.onLoadDone_(result, queryStartTime, queryEndTime);
            });
            this.ajax_.fail(this.onLoadFail_.bind(this));
        };
        ;
        ShipTrackLoader.prototype.mergeUndetermined_ = function (uid, pt) {
            var ret = [pt];
            var arr = this.undeterminedPoints_[uid];
            if (!arr)
                return ret;
            var last = pt;
            for (var i = arr.length - 1; i >= 0; i--) {
                var pti = arr[i];
                if (this.distance_(pti, pt) < 0.0001) {
                    ret.unshift(pti);
                    last = pti;
                }
            }
            return ret;
        };
        ;
        ShipTrackLoader.prototype.onLoadDone_ = function (result, start, end) {
            //console.log(result);
            this.ajax_ = null;
            if (result && result.length > 0) {
                for (var j = 0; j < result.length; j++) {
                    if (!result[j].ShipTrackSignals)
                        continue;
                    if (!this.points[result[j].ShipID])
                        this.points[result[j].ShipID] = [];
                    for (var i = 0; i < result[j].ShipTrackSignals.length; i++) {
                        var pt = result[j].ShipTrackSignals[i];
                        var uid = pt.ID;
                        var arr = this.points[uid];
                        if (!arr) {
                            this.points[uid] = this.mergeUndetermined_(uid, pt);
                        }
                        else if (arr.length == 0 || this.distance_(arr[arr.length - 1], pt) < 0.1) {
                            arr.push(pt);
                        }
                        else {
                            var arr = this.undeterminedPoints_[uid];
                            if (!arr)
                                this.undeterminedPoints_[uid] = [pt];
                            else
                                arr.push(pt);
                        }
                    }
                }
            }
            else {
                if (this.skipFun_)
                    this.skipFun_([[start, end]]);
            }
            utilities.notificationCenter.send('trackLoader/update', this.current_);
            if (this.current_ >= this.end_) {
                this.fireLoad_(true);
            }
            else {
                this.fireLoad_(false);
            }
        };
        ;
        ShipTrackLoader.prototype.distance_ = function (pt0, pt1) {
            var dt = pt0.Latitude - pt1.Latitude;
            var dn = pt0.Longitude - pt1.Longitude;
            return dt * dt + dn * dn;
        };
        ;
        ShipTrackLoader.prototype.onLoadFail_ = function (jqXHR, textStatus) {
            this.ajax_ = null;
            if (textStatus === 'abort')
                return;
            this.fireError_();
        };
        ;
        ShipTrackLoader.prototype.destroy = function () {
            if (this.ajax_) {
                this.ajax_.abort();
                this.ajax_ = null;
            }
            this.points = [];
            this.start_ = this.current_ = this.end_ = new Date(0);
        };
        ;
        ShipTrackLoader.prototype.fireLoad_ = function (finish) {
            if (this.fnLoad_)
                this.fnLoad_.apply(this.context_, [finish]);
        };
        ;
        ShipTrackLoader.prototype.fireError_ = function () {
            if (this.fnError_)
                this.fnError_.apply(this.context_);
        };
        ;
        return ShipTrackLoader;
    }());
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.default = ShipTrackLoader;
});
